# -*- coding: iso-8859-1 -*-

# deletePageWithAttributes.py --- Plugin XML-RPC command for MoinMoin deleting a page

# $Id: deletePageWithAttributes.py,v 1.3 2006/05/21 18:48:12 stefan Exp $

# Copyright 2006 Stefan Merten <smerten@oekonux.de>

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import xmlrpclib

from MoinMoin.PageEditor import PageEditor

###############################################################################
###############################################################################

def execute(xmlRpc, pagename, attributes):
    """
    Callback to execute the functionality of this plugin.

    Deletes a page using all the security mechanisms described for C{putPage}.
    In particular the configuration options C{xmlrpc_putpage_enabled} and
    C{xmlrpc_putpage_trusted_only} are considered.

    @param xmlRpc: Describes the local execution environment. While all other
                   parameters come from the remote caller this one is added by
                   the local execution environment.
    @type xmlRpc: XmlRpcBase

    @param pagename: The name of the page to delete.
    @type pagename: unicode | str

    @param attributes: Attributes describing the action further. See
                       http://www.merten-home.de/FreeSoftware/remoteMoin/manual.html#attributes
                       for the rationale.
    @type attributes: { 'comment': str | unicode, ..., }

    @return: Flag whether operation was successful or fault.
    @rtype: xmlrpclib.Boolean | xmlrpclib.Fault
    """
    # This is taken from `wikirpc.XmlRpcBase.xmlrpc_putPage` and modified. See
    # original `xmlrpc_putPage` comments for the configuration stuff.

    if xmlRpc.request.cfg.xmlrpc_putpage_enabled:
        pagename = xmlRpc._instr(pagename)
    else:
        pagename = u"PutPageTestPage"

    permissionDenied = xmlrpclib.Fault(1, "You are not allowed to delete this page")
    if (xmlRpc.request.cfg.xmlrpc_putpage_trusted_only
        and not xmlRpc.request.user.trusted):
        return permissionDenied
    if not xmlRpc.request.user.may.delete(pagename):
        return permissionDenied

    comment = xmlRpc._instr(attributes.get('comment', u''))
    page = PageEditor(xmlRpc.request, pagename)
    try:
        page.deletePage(comment)
    except OSError, error:
        return xmlrpclib.Fault(1, "Deletion failed: %s" % ( error, ))

    return xmlrpclib.Boolean(1)
