# -*- coding: iso-8859-1 -*-

# putPageWithAttributes.py --- Plugin XML-RPC command for MoinMoin putting a page with attributes

# $Id: putPageWithAttributes.py,v 1.2 2006/05/21 19:11:05 stefan Exp $

# Copyright 2006 Stefan Merten <smerten@oekonux.de>

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import xmlrpclib, StringIO

from MoinMoin.PageEditor import PageEditor

###############################################################################
###############################################################################

def execute(xmlRpc, pagename, pagetext, attributes):
    """
    Callback to execute the functionality of this plugin.

    Puts a new version of a page using all the security mechanisms described
    for C{putPage}. In particular the configuration options
    C{xmlrpc_putpage_enabled} and C{xmlrpc_putpage_trusted_only} are
    considered.

    @param xmlRpc: Describes the local execution environment. While all other
                   parameters come from the remote caller this one is added by
                   the local execution environment.
    @type xmlRpc: XmlRpcBase

    @param pagename: The name of the page to write.
    @type pagename: unicode | str

    @param pagetext: The new content of the page.
    @type pagetext: unicode | str

    @param attributes: Attributes describing the action further. See
                       http://www.merten-home.de/FreeSoftware/remoteMoin/manual.html#attributes
                       for the rationale.
    @type attributes: { 'comment': str | unicode,
                        'minoredit': xmlrpclib.Boolean, ..., }

    @return: Flag whether operation was successful or fault.
    @rtype: xmlrpclib.Boolean | xmlrpclib.Fault
    """
    # This is taken from `wikirpc.XmlRpcBase.xmlrpc_putPage` and modified. See
    # original `xmlrpc_putPage` comments for the configuration stuff.

    if xmlRpc.request.cfg.xmlrpc_putpage_enabled:
        pagename = xmlRpc._instr(pagename)
    else:
        pagename = u"PutPageTestPage"

    permissionDenied = xmlrpclib.Fault(1, "You are not allowed to edit this page")
    if (xmlRpc.request.cfg.xmlrpc_putpage_trusted_only
        and not xmlRpc.request.user.trusted):
        return permissionDenied
    if not xmlRpc.request.user.may.write(pagename):
        return permissionDenied

    comment = xmlRpc._instr(attributes.get('comment', u''))
    minoredit = attributes.get('minoredit', 0)
    if xmlRpc.version == 2:
        newtext = xmlRpc._instr(pagetext)
    elif xmlRpc.version == 1:
        newtext = xmlRpc._inlob(pagetext)
    page = PageEditor(xmlRpc.request, pagename)
    try:
        page.saveText(newtext, 0, trivial=minoredit, comment=comment)
    except page.SaveError, error:
        return xmlrpclib.Fault(1, "Saving failed: %s" % ( error, ))

    # we need this to update pagelinks cache
    out = StringIO.StringIO()
    xmlRpc.request.redirect(out)
    xmlRpc.request.args = xmlRpc.request.form = xmlRpc.request.setup_args({})
    page.send_page(xmlRpc.request, content_only=1)
    xmlRpc.request.redirect()

    return xmlrpclib.Boolean(1)
